@@ -1,6 +1,6 @@
 /* This is dvipdfmx, an eXtended version of dvipdfm by Mark A. Wicks.
 
-    Copyright (C) 2002-2016 by Jin-Hwan Cho and Shunsaku Hirata,
+    Copyright (C) 2002-2019 by Jin-Hwan Cho and Shunsaku Hirata,
     the dvipdfmx project team.
     
     Copyright (C) 1998, 1999 by Mark A. Wicks <mwicks@kettering.edu>
@@ -31,6 +31,7 @@
 #include "mem.h"
 #include "error.h"
 
+#include "dpxconf.h"
 #include "dpxfile.h"
 
 #include "pdfdoc.h"
@@ -38,13 +39,6 @@
 
 #include "pdfcolor.h"
 
-static int verbose = 0;
-void
-pdf_color_set_verbose (void)
-{
-  verbose++;
-}
-
 /* This function returns PDF_COLORSPACE_TYPE_GRAY,
  * PDF_COLORSPACE_TYPE_RGB, PDF_COLORSPACE_TYPE_CMYK or
  * PDF_COLORSPACE_TYPE_SPOT.
@@ -351,7 +345,7 @@ pdf_color_set (pdf_color *sc, pdf_color *fc)
 {
   pdf_color_copycolor(&color_stack.stroke[color_stack.current], sc);
   pdf_color_copycolor(&color_stack.fill[color_stack.current], fc);
-  pdf_dev_reset_color(0);
+  pdf_dev_reset_color(1);
 }
 
 void
@@ -373,7 +367,7 @@ pdf_color_pop (void)
     WARN("Color stack underflow. Just ignore.");
   } else {
     color_stack.current--;
-    pdf_dev_reset_color(0);
+    pdf_dev_reset_color(1);
   }
   return;
 }
@@ -596,19 +590,22 @@ static struct
   {0x04, 0x00}, /* PDF-1.5 */
   {0x04, 0x00}, /* PDF-1.6 */
   {0x04, 0x20}, /* PDF-1.7 */
+  {0x04, 0x20}, /* Dummy(1.8)*/
+  {0x04, 0x20}, /* Dummy(1.9) */
+  {0x04, 0x20}  /* PDF-2.0 */
 };
 
 static int
 iccp_version_supported (int major, int minor)
 {
-  int  pdf_ver;
+  int  idx;
 
-  pdf_ver = pdf_get_version();
-  if (pdf_ver < 8) {
-    if (icc_versions[pdf_ver].major < major)
+  idx = pdf_get_version() - 10;
+  if (idx < 11) {
+    if (icc_versions[idx].major < major)
       return 0;
-    else if (icc_versions[pdf_ver].major == major &&
-             icc_versions[pdf_ver].minor <  minor)
+    else if (icc_versions[idx].major == major &&
+             icc_versions[idx].minor <  minor)
       return 0;
     else {
       return 1;
@@ -1179,12 +1176,12 @@ iccp_load_profile (const char *ident,
   cspc_id = pdf_colorspace_findresource(ident,
 					PDF_COLORSPACE_TYPE_ICCBASED, cdata);
   if (cspc_id >= 0) {
-    if (verbose)
+    if (dpx_conf.verbose_level > 0)
       MESG("(ICCP:[id=%d])", cspc_id);
     release_iccbased_cdata(cdata);
     return cspc_id;
   }
-  if (verbose > 1) {
+  if (dpx_conf.verbose_level > 1) {
     print_iccp_header(&icch, checksum);
   }
 
@@ -1351,13 +1348,13 @@ pdf_colorspace_load_ICCBased (const char *ident, const char *filename)
   cspc_id = pdf_colorspace_findresource(ident,
 					PDF_COLORSPACE_TYPE_ICCBASED, cdata);
   if (cspc_id >= 0) {
-    if (verbose)
+    if (dpx_conf.verbose_level > 0)
       MESG("(ICCP:[id=%d])", cspc_id);
     release_iccbased_cdata(cdata);
     pdf_release_obj(stream);
     return cspc_id;
   }
-  if (verbose > 1) {
+  if (dpx_conf.verbose_level > 1) {
     print_iccp_header(&icch, checksum);
   }
 
@@ -1503,9 +1500,9 @@ pdf_colorspace_defineresource (const char *ident,
   colorspace->cdata    = cdata;
   colorspace->resource = resource;
 
-  if (verbose) {
+  if (dpx_conf.verbose_level > 0) {
     MESG("(ColorSpace:%s", ident);
-    if (verbose > 1) {
+    if (dpx_conf.verbose_level > 1) {
       switch (subtype) {
       case PDF_COLORSPACE_TYPE_ICCBASED:
 	MESG("[ICCBased]");
