@@ -520,6 +520,9 @@ pdf_doc_get_page_resources (pdf_doc *p, const char *category)
   if (!resources) {
     resources = pdf_new_dict();
     pdf_add_dict(res_dict, pdf_new_name(category), resources);
+  } else if (pdf_obj_typeof(resources) == PDF_INDIRECT) {
+    resources = pdf_deref_obj(resources); /* FIXME: deref_obj increment link count */
+    pdf_release_obj(resources); /* FIXME: just to decrement link count */
   }
 
   return resources;
@@ -533,10 +536,12 @@ pdf_doc_add_page_resource (const char *category,
   pdf_obj *resources;
   pdf_obj *duplicate;
 
+#if 0
   if (!PDF_OBJ_INDIRECTTYPE(resource_ref)) {
     WARN("Passed non indirect reference...");
     resource_ref = pdf_ref_obj(resource_ref); /* leak */
   }
+#endif
   resources = pdf_doc_get_page_resources(p, category);
   duplicate = pdf_lookup_dict(resources, resource_name);
   if (duplicate && pdf_compare_reference(duplicate, resource_ref)) {
@@ -2545,9 +2550,10 @@ pdf_open_document (const char *filename,
   if (manual_thumb_enabled) {
     if (strlen(filename) > 4 &&
         !strncmp(".pdf", filename + strlen(filename) - 4, 4)) {
-      thumb_basename = NEW(strlen(filename)-4+1, char);
-      strncpy(thumb_basename, filename, strlen(filename)-4);
-      thumb_basename[strlen(filename)-4] = 0;
+      size_t len = strlen(filename) - strlen(".pdf");
+      thumb_basename = NEW(len+1, char);
+      strncpy(thumb_basename, filename, len);
+      thumb_basename[len] = 0;
     } else {
       thumb_basename = NEW(strlen(filename)+1, char);
       strcpy(thumb_basename, filename);
