@@ -1,6 +1,6 @@
 /* This is dvipdfmx, an eXtended version of dvipdfm by Mark A. Wicks.
 
-    Copyright (C) 2002-2016 by Jin-Hwan Cho and Shunsaku Hirata,
+    Copyright (C) 2002-2018 by Jin-Hwan Cho and Shunsaku Hirata,
     the dvipdfmx project team.
     
     Copyright (C) 1998, 1999 by Mark A. Wicks <mwicks@kettering.edu>
@@ -25,8 +25,6 @@
 
 #include "numbers.h"
 
-extern void tfm_set_verbose (void);
-
 extern int  tfm_open (const char * tex_name, int must_exist);
 extern void tfm_close_all (void);
 
@@ -52,6 +50,7 @@ extern double tfm_get_design_size  (int font_id);
 extern char  *tfm_get_codingscheme (int font_id);
 extern int tfm_is_vert (int font_id);
 #endif
+extern int tfm_is_jfm (int font_id);
 
 extern int tfm_exists  (const char *tfm_name);
 
