@@ -2,7 +2,7 @@
 ** CMap.hpp                                                             **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2018 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2020 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -28,9 +28,7 @@
 #include "RangeMap.hpp"
 
 
-struct CMap : public NamedFontEncoding
-{
-	virtual ~CMap () =default;
+struct CMap : public NamedFontEncoding {
 	virtual bool vertical () const =0;
 	virtual bool mapsToCID () const =0;
 	virtual uint32_t cid (uint32_t c) const =0;
@@ -48,8 +46,7 @@ struct CMap : public NamedFontEncoding
 };
 
 
-struct IdentityCMap : public CMap
-{
+struct IdentityCMap : public CMap {
 	uint32_t cid (uint32_t c) const override      {return c;}
 	uint32_t bfcode (uint32_t cid) const override {return 0;}
 	std::string getROString () const override     {return "Adobe-Identity";}
@@ -57,38 +54,34 @@ struct IdentityCMap : public CMap
 };
 
 
-struct IdentityHCMap : public IdentityCMap
-{
+struct IdentityHCMap : public IdentityCMap {
 	bool vertical () const override    {return false;}
 	const char* name () const override {return "Identity-H";}
 };
 
 
-struct IdentityVCMap : public IdentityCMap
-{
+struct IdentityVCMap : public IdentityCMap {
 	bool vertical () const override    {return true;}
 	const char* name () const override {return "Identity-V";}
 };
 
 
-struct UnicodeCMap : public CMap
-{
+struct UnicodeCMap : public CMap {
 	bool vertical () const override               {return false;}
 	const char* name () const override            {return "unicode";}
 	bool mapsToCID () const override              {return false;}
-	const char* path () const override            {return 0;}
+	const char* path () const override            {return nullptr;}
 	uint32_t cid (uint32_t c) const override      {return c;}
 	uint32_t bfcode (uint32_t cid) const override {return cid;}
 	std::string getROString () const override     {return "";}
 };
 
 
-class SegmentedCMap : public CMap
-{
+class SegmentedCMap : public CMap {
 	friend class CMapReader;
 
 	public:
-		SegmentedCMap (const std::string &fname) : _filename(fname), _basemap(0), _vertical(false), _mapsToCID(true) {}
+		explicit SegmentedCMap (std::string fname) : _filename(std::move(fname)) {}
 		const char* name () const override {return _filename.c_str();}
 		uint32_t cid (uint32_t c) const override;
 		uint32_t bfcode (uint32_t cid) const override;
@@ -105,9 +98,9 @@ class SegmentedCMap : public CMap
 		std::string _filename;
 		std::string _registry;
 		std::string _ordering;
-		CMap *_basemap;
-		bool _vertical;
-		bool _mapsToCID;   // true: chrcode->CID, false: CID->charcode
+		CMap *_basemap = nullptr;
+		bool _vertical = false;
+		bool _mapsToCID = true;   // true: chrcode->CID, false: CID->charcode
 		RangeMap _cidranges;
 		RangeMap _bfranges;
 };
