@@ -2,7 +2,7 @@
 ** CMapReader.cpp                                                       **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2019 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2020 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -32,10 +32,6 @@
 using namespace std;
 
 
-CMapReader::CMapReader () : _inCMap(false) {
-}
-
-
 /** Reads a cmap file and returns the corresponding CMap object.
  *  @param fname[in] name/path of cmap file
  *  @return CMap object representing the read data, or 0 if file could not be read */
@@ -74,7 +70,7 @@ unique_ptr<CMap> CMapReader::read (std::istream& is, const string &name) {
 		}
 	}
 	catch (CMapReaderException &e) {
-		_cmap.release();
+		_cmap.reset();
 		throw;
 	}
 	return std::move(_cmap);
@@ -89,9 +85,10 @@ void CMapReader::executeOperator (const string &opname, InputReader &ir) {
 		const char *name;
 		void (CMapReader::*handler)(InputReader&);
 	};
-	constexpr array<Operator, 6> operators {{
+	constexpr array<Operator, 7> operators {{
 		{"beginbfchar",   &CMapReader::op_beginbfchar},
 		{"beginbfrange",  &CMapReader::op_beginbfrange},
+		{"begincidchar",  &CMapReader::op_begincidchar},
 		{"begincidrange", &CMapReader::op_begincidrange},
 		{"def",           &CMapReader::op_def},
 		{"endcmap",       &CMapReader::op_endcmap},
@@ -135,7 +132,7 @@ void CMapReader::op_usecmap (InputReader &) {
 		throw CMapReaderException("stack underflow while processing usecmap");
 	else {
 		const string name = popToken().strvalue();
-		if ((_cmap->_basemap = CMapManager::instance().lookup(name)) == 0)
+		if ((_cmap->_basemap = CMapManager::instance().lookup(name)) == nullptr)
 			throw CMapReaderException("CMap file '"+name+"' not found");
 	}
 }
@@ -154,17 +151,19 @@ static uint32_t parse_hexentry (InputReader &ir) {
 }
 
 
-void CMapReader::op_begincidrange (InputReader &ir) {
+void CMapReader::parseCIDChars (InputReader &ir, bool isRange) {
 	if (!_tokens.empty() && _tokens.back().type() == Token::Type::NUMBER) {
 		ir.skipSpace();
 		int num_entries = static_cast<int>(popToken().numvalue());
 		while (num_entries > 0 && ir.peek() == '<') {
 			uint32_t first = parse_hexentry(ir);
-			uint32_t last = parse_hexentry(ir);
-			uint32_t cid;
+			uint32_t last = first;
+			if (isRange)
+				last = parse_hexentry(ir);
 			ir.skipSpace();
+			uint32_t cid;
 			if (!ir.parseUInt(cid))
-				throw CMapReaderException("invalid range entry (decimal value expected)");
+				throw CMapReaderException("invalid char entry (decimal value expected)");
 			_cmap->addCIDRange(first, last, cid);
 			ir.skipSpace();
 		}
@@ -172,6 +171,16 @@ void CMapReader::op_begincidrange (InputReader &ir) {
 }
 
 
+void CMapReader::op_begincidchar (InputReader &ir) {
+	parseCIDChars(ir, false);
+}
+
+
+void CMapReader::op_begincidrange (InputReader &ir) {
+	parseCIDChars(ir, true);
+}
+
+
 void CMapReader::op_beginbfrange (InputReader &ir) {
 	if (!_tokens.empty() && _tokens.back().type() == Token::Type::NUMBER) {
 		ir.skipSpace();
@@ -249,9 +258,7 @@ void CMapReader::Token::scan (InputReader &ir) {
 	else if (isdigit(ir.peek())) {  // number?
 		double val;
 		if (ir.parseDouble(val)) {
-			ostringstream oss;
-			oss << val;
-			_value = oss.str();
+			_value = util::to_string(val);
 			_type = Type::NUMBER;
 		}
 	}
