@@ -2,7 +2,7 @@
 ** DvisvgmSpecialHandler.hpp                                            **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2019 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2020 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -21,6 +21,7 @@
 #ifndef DVISVGMSPECIALHANDLER_HPP
 #define DVISVGMSPECIALHANDLER_HPP
 
+#include <memory>
 #include <string>
 #include <unordered_map>
 #include <vector>
@@ -28,8 +29,44 @@
 
 class InputReader;
 class SpecialActions;
+class SVGTree;
+class XMLElement;
+class XMLNode;
+
+#ifdef _MSC_VER
+// MSVC: Prevent aggressive optimization of pointers to member functions.
+// Instatiating class DvisvgmSpecialHandler without the following pragma
+// leads to memory corruption.
+// https://docs.microsoft.com/en-us/cpp/preprocessor/pointers-to-members
+#pragma pointers_to_members(full_generality, single_inheritance)
+#endif
 
 class DvisvgmSpecialHandler : public SpecialHandler {
+	class XMLParser {
+		using AppendFunc = void (SVGTree::*)(std::unique_ptr<XMLNode>);
+		using PushFunc = void (SVGTree::*)(std::unique_ptr<XMLElement>);
+		using PopFunc = void (SVGTree::*)();
+		using NameStack = std::vector<std::string>;
+
+		public:
+			XMLParser (AppendFunc append, PushFunc push, PopFunc pop)
+				: _append(append), _pushContext(push), _popContext(pop) {}
+
+			void parse (const std::string &xml, SpecialActions &actions, bool finish=false);
+			void flush (SpecialActions &actions);
+
+		protected:
+			void openElement (const std::string &tag, SpecialActions &actions);
+			void closeElement (const std::string &tag, SpecialActions &actions);
+
+		private:
+			AppendFunc _append;
+			PushFunc _pushContext;
+			PopFunc _popContext;
+			std::string _xmlbuf;
+			NameStack _nameStack;  ///< names of nested elements still missing a closing tag
+	};
+
 	using StringVector = std::vector<std::string>;
 	using MacroMap = std::unordered_map<std::string, StringVector>;
 
@@ -60,7 +97,9 @@ class DvisvgmSpecialHandler : public SpecialHandler {
 	private:
 		MacroMap _macros;
 		MacroMap::iterator _currentMacro;
-		int _nestingLevel;
+		int _nestingLevel=0;    ///< nesting depth of rawset specials
+		XMLParser _defsParser;  ///< parses XML added by 'rawdef' specials
+		XMLParser _pageParser;  ///< parses XML added by 'raw' specials
 };
 
 #endif
