@@ -2,7 +2,7 @@
 ** FontMap.hpp                                                          **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2019 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2020 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -34,7 +34,7 @@ class Subfont;
 class FontMap {
 	public:
 		struct Entry {
-			Entry (const MapLine &mapline, Subfont *subfont=nullptr);
+			explicit Entry (const MapLine &mapline, Subfont *subfont=nullptr);
 			Entry (const Entry &entry) =delete;
 			Entry (Entry &&entry) =default;
 			Entry& operator = (Entry &&entry) =default;
