@@ -2,7 +2,7 @@
 ** HashFunction.cpp                                                     **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2019 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2020 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -31,7 +31,12 @@ using namespace std;
 
 /** Returns a vector containing the names of the currently supported hash algorithms. */
 vector<string> HashFunction::supportedAlgorithms () {
-	return vector<string> {"md5", "xxh32", "xxh64"};
+	return vector<string> {
+		"md5", "xxh32", "xxh64",
+#ifdef ENABLE_XXH128
+		"xxh128"
+#endif
+	};
 }
 
 
@@ -52,6 +57,10 @@ unique_ptr<HashFunction> HashFunction::create (const string &name) {
 		return util::make_unique<XXH32HashFunction>();
 	if (lowerName == "xxh64")
 		return util::make_unique<XXH64HashFunction>();
+#ifdef ENABLE_XXH128
+	if (lowerName == "xxh128")
+		return util::make_unique<XXH128HashFunction>();
+#endif
 	return nullptr;
 }
 
@@ -74,6 +83,15 @@ std::unique_ptr<HashFunction> HashFunction::create (const string &name, const ve
 }
 
 
+void HashFunction::update (istream &is) {
+	char buf[4096];
+	while (is) {
+		is.read(buf, 4096);
+		update(buf, is.gcount());
+	}
+}
+
+
 /** Returns the current digest as hexadecimal value. */
 string HashFunction::digestString () const {
 	ostringstream oss;
