@@ -2,7 +2,7 @@
 ** PreScanDVIReader.cpp                                                 **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2017 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2020 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -25,7 +25,7 @@ using namespace std;
 
 
 PreScanDVIReader::PreScanDVIReader (std::istream &is, DVIActions *actions)
-	: BasicDVIReader(is), _actions(actions), _currentPageNumber(0)
+	: BasicDVIReader(is), _actions(actions)
 {
 }
 
