@@ -2,7 +2,7 @@
 ** Process.cpp                                                          **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2019 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2020 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -21,9 +21,10 @@
 #ifdef _WIN32
 	#include "windows.hpp"
 #else
+	#include <csignal>
+	#include <cstring>
 	#include <fcntl.h>
 	#include <sys/wait.h>
-	#include <signal.h>
 	#include <unistd.h>
 #endif
 
@@ -42,7 +43,7 @@ class Subprocess {
 		enum class State {RUNNING, FINISHED, FAILED};
 
 	public:
-		Subprocess ();
+		Subprocess () =default;
 		Subprocess (const Subprocess&) =delete;
 		Subprocess (Subprocess&&) =delete;
 		~Subprocess ();
@@ -52,17 +53,17 @@ class Subprocess {
 
 	private:
 #ifdef _WIN32
-		HANDLE _pipeReadHandle;   ///< handle of read end of pipe
-		HANDLE _childProcHandle;  ///< handle of child process
+		HANDLE _pipeReadHandle=NULL;   ///< handle of read end of pipe
+		HANDLE _childProcHandle=NULL;  ///< handle of child process
 #else
-		int _readfd; ///< file descriptor of read end of pipe
-		pid_t _pid;  ///< PID of the subprocess
+		int _readfd=-1; ///< file descriptor of read end of pipe
+		pid_t _pid=-1;  ///< PID of the subprocess
 #endif
 };
 
 
-Process::Process (const string &cmd, const string &paramstr)
-	: _cmd(cmd), _paramstr(paramstr)
+Process::Process (string cmd, string paramstr)
+	: _cmd(std::move(cmd)), _paramstr(std::move(paramstr))
 {
 }
 
@@ -113,10 +114,6 @@ static inline void close_and_zero_handle (HANDLE &handle) {
 }
 
 
-Subprocess::Subprocess() : _pipeReadHandle(NULL), _childProcHandle(NULL) {
-}
-
-
 Subprocess::~Subprocess () {
 	if (_pipeReadHandle != NULL)
 		CloseHandle(_pipeReadHandle);
@@ -214,10 +211,6 @@ Subprocess::State Subprocess::state () {
 
 #else  // !_WIN32
 
-Subprocess::Subprocess () : _readfd(-1), _pid(-1) {
-}
-
-
 Subprocess::~Subprocess () {
 	if (_readfd >= 0)
 		close(_readfd);
@@ -276,12 +269,12 @@ static void split_paramstr (string &paramstr, vector<const char*> &params) {
 
 
 /** Starts a child process.
- *  @param[in] cmd name of command to execute
- *  @param[in] paramstr parameters required by command
+ *  @param[in] cmd name of command to execute or absolute path to executable
+ *  @param[in] paramstr parameters required by the command
  *  @returns true if child process started properly */
 bool Subprocess::run (const string &cmd, string paramstr) {
 	int pipefd[2];
-	if (pipe(pipefd) < 0)
+	if (cmd.empty() || pipe(pipefd) < 0)
 		return false;
 
 	_pid = fork();
@@ -299,9 +292,11 @@ bool Subprocess::run (const string &cmd, string paramstr) {
 		vector<const char*> params;
 		params.push_back(cmd.c_str());
 		split_paramstr(paramstr, params);
-		params.push_back(nullptr); // trailing null pointer marks end of parameter list
-		signal(SIGINT, SIG_IGN);   // child process is supposed to ignore ctrl-c events
-		execvp(cmd.c_str(), const_cast<char* const*>(&params[0]));
+		params.push_back(nullptr);  // trailing null pointer marks end of parameter list
+		signal(SIGINT, SIG_IGN);    // child process is supposed to ignore ctrl-c events
+		if (params[0][0] == '/')    // absolute path to executable?
+			params[0] = strrchr(params[0], '/')+1;  // filename of executable
+		execvp(cmd.c_str(), const_cast<char* const*>(params.data()));
 		exit(1);
 	}
 	_readfd = pipefd[0];
