@@ -2,7 +2,7 @@
 ** SpecialActions.hpp                                                   **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2018 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2020 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -25,10 +25,12 @@
 #include <memory>
 #include "BoundingBox.hpp"
 #include "Color.hpp"
+#include "FilePath.hpp"
 #include "Matrix.hpp"
+#include "SVGTree.hpp"
 
+class XMLElement;
 class XMLNode;
-class XMLElementNode;
 
 class SpecialActions {
 	public:
@@ -44,20 +46,20 @@ class SpecialActions {
 		virtual const Matrix& getMatrix () const =0;
 		virtual Matrix getPageTransformation () const {return Matrix(1);}
 		virtual void setBgColor (const Color &color) =0;
-		virtual void appendToPage(std::unique_ptr<XMLNode> &&node) =0;
-		virtual void appendToDefs(std::unique_ptr<XMLNode> &&node) =0;
-		virtual void prependToPage(std::unique_ptr<XMLNode> &&node) =0;
-		virtual void pushContextElement (std::unique_ptr<XMLElementNode> &&node) =0;
-		virtual void popContextElement () =0;
+		virtual const SVGTree& svgTree () const =0;
+		SVGTree& svgTree () {return const_cast<SVGTree&>(const_cast<const SpecialActions*>(this)->svgTree());}
 		virtual BoundingBox& bbox () =0;
 		virtual BoundingBox& bbox (const std::string &name, bool reset=false) =0;
 		virtual void embed (const BoundingBox &bbox) =0;
 		virtual void embed (const DPair &p, double r=0) =0;
 		virtual unsigned getCurrentPageNumber () const =0;
-		virtual std::string getSVGFilename (unsigned pageno) const =0;
+		virtual FilePath getSVGFilePath (unsigned pageno) const =0;
 		virtual std::string getBBoxFormatString () const =0;
 		virtual void progress (const char *id) {}
 		virtual int getDVIStackDepth () const  {return 0;}
+		virtual void lockOutput () {}
+		virtual void unlockOutput () {}
+		virtual bool outputLocked () const {return false;}
 
 		static double PROGRESSBAR_DELAY; ///< progress bar doesn't appear before this time has elapsed (in sec)
 };
@@ -65,6 +67,7 @@ class SpecialActions {
 
 class EmptySpecialActions : public SpecialActions {
 	public:
+		EmptySpecialActions () : _matrix(1) {_svg.newPage(1);}
 		double getX () const override {return 0;}
 		double getY () const override {return 0;}
 		void setX (double x) override {}
@@ -75,22 +78,19 @@ class EmptySpecialActions : public SpecialActions {
 		Color getColor () const override {return Color::BLACK;}
 		void setMatrix (const Matrix &m) override {}
 		const Matrix& getMatrix () const override {return _matrix;}
-		void appendToPage(std::unique_ptr<XMLNode> &&node) override {}
-		void appendToDefs(std::unique_ptr<XMLNode> &&node) override {}
-		void prependToPage(std::unique_ptr<XMLNode> &&node) override {}
-		void pushContextElement (std::unique_ptr<XMLElementNode> &&node) override {}
-		void popContextElement () override {}
+		const SVGTree& svgTree () const override {return _svg;}
 		BoundingBox& bbox () override {return _bbox;}
 		BoundingBox& bbox (const std::string &name, bool reset=false) override {return _bbox;}
 		void embed (const BoundingBox &bbox) override {}
 		void embed (const DPair &p, double r=0) override {}
 		unsigned getCurrentPageNumber() const override {return 0;}
-		std::string getSVGFilename (unsigned pageno) const override {return "";}
+		FilePath getSVGFilePath (unsigned pageno) const override {return FilePath();}
 		std::string getBBoxFormatString () const override {return "";}
 
 	private:
 		BoundingBox _bbox;
 		Matrix _matrix;
+		SVGTree _svg;
 };
 
 #endif
