@@ -2,7 +2,7 @@
 ** SpecialHandler.hpp                                                   **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2018 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2020 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -32,7 +32,7 @@ class SpecialManager;
 
 
 struct SpecialException : public MessageException {
-	SpecialException (const std::string &msg) : MessageException(msg) {}
+	explicit SpecialException (const std::string &msg) : MessageException(msg) {}
 };
 
 
