@@ -2,7 +2,7 @@
 ** DVIReaderTest.cpp                                                    **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2019 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2020 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -175,7 +175,7 @@ static void test_dvi_file (const char *fname, const vector<const char*> &expecte
 		ASSERT_FALSE(ss.eof());
 		ss.getline(line, 512);
 		lineno++;
-		ASSERT_EQ(line, expected_line) << "log line #" << lineno;
+		EXPECT_EQ(line, expected_line) << "log line #" << lineno;
 	}
 	ss.getline(line, 512);
 	EXPECT_EQ(*line, '\0');
@@ -241,7 +241,7 @@ TEST(DVIReaderTest, dvi_v2) {
 		"setchar0 103 [h=174.789, v=62.765, x=0, y=0, w=3.321, z=0, d=0]",
 		"setchar0 101 [h=179.217, v=62.765, x=0, y=0, w=3.321, z=0, d=0]",
 		"setchar0 110 [h=184.752, v=62.765, x=0, y=0, w=3.321, z=0, d=0]",
-		"right -0.277 [h=184.475, v=62.765, x=0, y=0, w=3.321, z=0, d=0]",
+		"right -.277 [h=184.475, v=62.765, x=0, y=0, w=3.321, z=0, d=0]",
 		"setchar0 116 [h=188.35, v=62.765, x=0, y=0, w=3.321, z=0, d=0]",
 		"setchar0 97 [h=193.331, v=62.765, x=0, y=0, w=3.321, z=0, d=0]",
 		"xxx 'color pop' [h=193.331, v=62.765, x=0, y=0, w=3.321, z=0, d=0]",
@@ -287,7 +287,7 @@ TEST(DVIReaderTest, dvi_v2) {
 		"pop [h=143.822, v=107.402, x=0, y=0, w=3.321, z=0, d=0]",
 		"right 37.446 [h=181.268, v=107.402, x=0, y=0, w=3.321, z=0, d=0]",
 		"setchar0 119 [h=188.463, v=107.402, x=0, y=0, w=3.321, z=0, d=0]",
-		"right -0.277 [h=188.187, v=107.402, x=0, y=0, w=3.321, z=0, d=0]",
+		"right -.277 [h=188.187, v=107.402, x=0, y=0, w=3.321, z=0, d=0]",
 		"setchar0 111 [h=193.168, v=107.402, x=0, y=0, w=3.321, z=0, d=0]",
 		"setchar0 114 [h=197.07, v=107.402, x=0, y=0, w=3.321, z=0, d=0]",
 		"setchar0 100 [h=202.605, v=107.402, x=0, y=0, w=3.321, z=0, d=0]",
@@ -298,7 +298,7 @@ TEST(DVIReaderTest, dvi_v2) {
 		"push [h=61.768, v=160.275, x=0, y=0, w=0, z=0, d=0]",
 		"push [h=61.768, v=160.275, x=0, y=0, w=0, z=0, d=0]",
 		"down -31.154 [h=61.768, v=129.121, x=0, y=0, w=0, z=0, d=0]",
-		"putrule 0.797, 220.922 [h=61.768, v=129.121, x=0, y=0, w=0, z=0, d=0]",
+		"putrule .797, 220.922 [h=61.768, v=129.121, x=0, y=0, w=0, z=0, d=0]",
 		"y 11.157 [h=61.768, v=140.278, x=0, y=11.157, w=0, z=0, d=0]",
 		"push [h=61.768, v=140.278, x=0, y=11.157, w=0, z=0, d=0]",
 		"push [h=61.768, v=140.278, x=0, y=11.157, w=0, z=0, d=0]",
@@ -333,7 +333,7 @@ TEST(DVIReaderTest, dvi_v2) {
 		"pop [h=61.768, v=140.278, x=0, y=11.157, w=0, z=0, d=0]",
 		"pop [h=61.768, v=140.278, x=0, y=11.157, w=0, z=0, d=0]",
 		"down 5.8 [h=61.768, v=146.078, x=0, y=11.157, w=0, z=0, d=0]",
-		"putrule 0.498, 220.922 [h=61.768, v=146.078, x=0, y=11.157, w=0, z=0, d=0]",
+		"putrule .498, 220.922 [h=61.768, v=146.078, x=0, y=11.157, w=0, z=0, d=0]",
 		"y0 [h=61.768, v=157.235, x=0, y=11.157, w=0, z=0, d=0]",
 		"push [h=61.768, v=157.235, x=0, y=11.157, w=0, z=0, d=0]",
 		"push [h=61.768, v=157.235, x=0, y=11.157, w=0, z=0, d=0]",
@@ -419,11 +419,11 @@ TEST(DVIReaderTest, dvi_v2) {
 		"setchar0 110 [h=115.401, v=181.145, x=0, y=11.955, w=3.321, z=0, d=0]",
 		"setchar0 111 [h=120.382, v=181.145, x=0, y=11.955, w=3.321, z=0, d=0]",
 		"setchar0 110 [h=125.917, v=181.145, x=0, y=11.955, w=3.321, z=0, d=0]",
-		"w -0.277 [h=125.64, v=181.145, x=0, y=11.955, w=-0.277, z=0, d=0]",
-		"setchar0 117 [h=131.175, v=181.145, x=0, y=11.955, w=-0.277, z=0, d=0]",
-		"setchar0 109 [h=139.477, v=181.145, x=0, y=11.955, w=-0.277, z=0, d=0]",
-		"w0 [h=139.2, v=181.145, x=0, y=11.955, w=-0.277, z=0, d=0]",
-		"setchar0 121 [h=144.458, v=181.145, x=0, y=11.955, w=-0.277, z=0, d=0]",
+		"w -.277 [h=125.64, v=181.145, x=0, y=11.955, w=-.277, z=0, d=0]",
+		"setchar0 117 [h=131.175, v=181.145, x=0, y=11.955, w=-.277, z=0, d=0]",
+		"setchar0 109 [h=139.477, v=181.145, x=0, y=11.955, w=-.277, z=0, d=0]",
+		"w0 [h=139.2, v=181.145, x=0, y=11.955, w=-.277, z=0, d=0]",
+		"setchar0 121 [h=144.458, v=181.145, x=0, y=11.955, w=-.277, z=0, d=0]",
 		"pop [h=61.768, v=181.145, x=0, y=11.955, w=0, z=0, d=0]",
 		"push [h=61.768, v=181.145, x=0, y=11.955, w=0, z=0, d=0]",
 		"right 110.558 [h=172.326, v=181.145, x=0, y=11.955, w=0, z=0, d=0]",
@@ -432,31 +432,31 @@ TEST(DVIReaderTest, dvi_v2) {
 		"setchar0 114 [h=183.423, v=181.145, x=0, y=11.955, w=0, z=0, d=0]",
 		"setchar0 109 [h=191.726, v=181.145, x=0, y=11.955, w=0, z=0, d=0]",
 		"setchar0 111 [h=196.707, v=181.145, x=0, y=11.955, w=0, z=0, d=0]",
-		"w 0.277 [h=196.984, v=181.145, x=0, y=11.955, w=0.277, z=0, d=0]",
-		"setchar0 100 [h=202.519, v=181.145, x=0, y=11.955, w=0.277, z=0, d=0]",
-		"x 3.321 [h=205.84, v=181.145, x=3.321, y=11.955, w=0.277, z=0, d=0]",
-		"setchar0 116 [h=209.714, v=181.145, x=3.321, y=11.955, w=0.277, z=0, d=0]",
-		"setchar0 101 [h=214.142, v=181.145, x=3.321, y=11.955, w=0.277, z=0, d=0]",
-		"setchar0 109 [h=222.444, v=181.145, x=3.321, y=11.955, w=0.277, z=0, d=0]",
-		"setchar0 112 [h=227.979, v=181.145, x=3.321, y=11.955, w=0.277, z=0, d=0]",
-		"w0 [h=228.256, v=181.145, x=3.321, y=11.955, w=0.277, z=0, d=0]",
-		"setchar0 111 [h=233.237, v=181.145, x=3.321, y=11.955, w=0.277, z=0, d=0]",
-		"setchar0 114 [h=237.139, v=181.145, x=3.321, y=11.955, w=0.277, z=0, d=0]",
-		"x0 [h=240.46, v=181.145, x=3.321, y=11.955, w=0.277, z=0, d=0]",
-		"setchar0 105 [h=243.227, v=181.145, x=3.321, y=11.955, w=0.277, z=0, d=0]",
-		"setchar0 110 [h=248.762, v=181.145, x=3.321, y=11.955, w=0.277, z=0, d=0]",
-		"w -0.277 [h=248.485, v=181.145, x=3.321, y=11.955, w=-0.277, z=0, d=0]",
-		"setchar0 118 [h=253.743, v=181.145, x=3.321, y=11.955, w=-0.277, z=0, d=0]",
-		"setchar0 105 [h=256.511, v=181.145, x=3.321, y=11.955, w=-0.277, z=0, d=0]",
-		"setchar0 100 [h=262.046, v=181.145, x=3.321, y=11.955, w=-0.277, z=0, d=0]",
-		"setchar0 117 [h=267.58, v=181.145, x=3.321, y=11.955, w=-0.277, z=0, d=0]",
-		"setchar0 110 [h=273.115, v=181.145, x=3.321, y=11.955, w=-0.277, z=0, d=0]",
-		"w0 [h=272.838, v=181.145, x=3.321, y=11.955, w=-0.277, z=0, d=0]",
-		"setchar0 116 [h=276.713, v=181.145, x=3.321, y=11.955, w=-0.277, z=0, d=0]",
+		"w .277 [h=196.984, v=181.145, x=0, y=11.955, w=.277, z=0, d=0]",
+		"setchar0 100 [h=202.519, v=181.145, x=0, y=11.955, w=.277, z=0, d=0]",
+		"x 3.321 [h=205.84, v=181.145, x=3.321, y=11.955, w=.277, z=0, d=0]",
+		"setchar0 116 [h=209.714, v=181.145, x=3.321, y=11.955, w=.277, z=0, d=0]",
+		"setchar0 101 [h=214.142, v=181.145, x=3.321, y=11.955, w=.277, z=0, d=0]",
+		"setchar0 109 [h=222.444, v=181.145, x=3.321, y=11.955, w=.277, z=0, d=0]",
+		"setchar0 112 [h=227.979, v=181.145, x=3.321, y=11.955, w=.277, z=0, d=0]",
+		"w0 [h=228.256, v=181.145, x=3.321, y=11.955, w=.277, z=0, d=0]",
+		"setchar0 111 [h=233.237, v=181.145, x=3.321, y=11.955, w=.277, z=0, d=0]",
+		"setchar0 114 [h=237.139, v=181.145, x=3.321, y=11.955, w=.277, z=0, d=0]",
+		"x0 [h=240.46, v=181.145, x=3.321, y=11.955, w=.277, z=0, d=0]",
+		"setchar0 105 [h=243.227, v=181.145, x=3.321, y=11.955, w=.277, z=0, d=0]",
+		"setchar0 110 [h=248.762, v=181.145, x=3.321, y=11.955, w=.277, z=0, d=0]",
+		"w -.277 [h=248.485, v=181.145, x=3.321, y=11.955, w=-.277, z=0, d=0]",
+		"setchar0 118 [h=253.743, v=181.145, x=3.321, y=11.955, w=-.277, z=0, d=0]",
+		"setchar0 105 [h=256.511, v=181.145, x=3.321, y=11.955, w=-.277, z=0, d=0]",
+		"setchar0 100 [h=262.046, v=181.145, x=3.321, y=11.955, w=-.277, z=0, d=0]",
+		"setchar0 117 [h=267.58, v=181.145, x=3.321, y=11.955, w=-.277, z=0, d=0]",
+		"setchar0 110 [h=273.115, v=181.145, x=3.321, y=11.955, w=-.277, z=0, d=0]",
+		"w0 [h=272.838, v=181.145, x=3.321, y=11.955, w=-.277, z=0, d=0]",
+		"setchar0 116 [h=276.713, v=181.145, x=3.321, y=11.955, w=-.277, z=0, d=0]",
 		"pop [h=61.768, v=181.145, x=0, y=11.955, w=0, z=0, d=0]",
 		"pop [h=61.768, v=181.145, x=0, y=11.955, w=0, z=0, d=0]",
 		"down 6.099 [h=61.768, v=187.245, x=0, y=11.955, w=0, z=0, d=0]",
-		"putrule 0.797, 220.922 [h=61.768, v=187.245, x=0, y=11.955, w=0, z=0, d=0]",
+		"putrule .797, 220.922 [h=61.768, v=187.245, x=0, y=11.955, w=0, z=0, d=0]",
 		"pop [h=61.768, v=160.275, x=0, y=0, w=0, z=0, d=0]",
 		"pop [h=61.768, v=160.275, x=0, y=0, w=0, z=0, d=0]",
 		"pop [h=61.768, v=160.275, x=0, y=0, w=0, z=0, d=0]",
@@ -473,9 +473,9 @@ TEST(DVIReaderTest, dvi_v2) {
 		"push [h=61.768, v=218.472, x=0, y=0, w=0, z=0, d=0]",
 		"right 3.387 [h=65.156, v=218.472, x=0, y=0, w=0, z=0, d=0]",
 		"setchar0 98 [h=70.69, v=218.472, x=0, y=0, w=0, z=0, d=0]",
-		"right 0.277 [h=70.967, v=218.472, x=0, y=0, w=0, z=0, d=0]",
+		"right .277 [h=70.967, v=218.472, x=0, y=0, w=0, z=0, d=0]",
 		"setchar0 111 [h=75.949, v=218.472, x=0, y=0, w=0, z=0, d=0]",
-		"right -0.277 [h=75.672, v=218.472, x=0, y=0, w=0, z=0, d=0]",
+		"right -.277 [h=75.672, v=218.472, x=0, y=0, w=0, z=0, d=0]",
 		"setchar0 120 [h=80.93, v=218.472, x=0, y=0, w=0, z=0, d=0]",
 		"setchar0 101 [h=85.358, v=218.472, x=0, y=0, w=0, z=0, d=0]",
 		"setchar0 100 [h=90.893, v=218.472, x=0, y=0, w=0, z=0, d=0]",
@@ -493,15 +493,15 @@ TEST(DVIReaderTest, dvi_v2) {
 		"push [h=61.768, v=218.472, x=0, y=0, w=0, z=0, d=0]",
 		"push [h=61.768, v=218.472, x=0, y=0, w=0, z=0, d=0]",
 		"down -9.907 [h=61.768, v=208.565, x=0, y=0, w=0, z=0, d=0]",
-		"putrule 0.398, 53.267 [h=61.768, v=208.565, x=0, y=0, w=0, z=0, d=0]",
+		"putrule .398, 53.267 [h=61.768, v=208.565, x=0, y=0, w=0, z=0, d=0]",
 		"down 13.095 [h=61.768, v=221.66, x=0, y=0, w=0, z=0, d=0]",
 		"push [h=61.768, v=221.66, x=0, y=0, w=0, z=0, d=0]",
-		"setrule 13.295, 0.398 [h=62.167, v=221.66, x=0, y=0, w=0, z=0, d=0]",
+		"setrule 13.295, .398 [h=62.167, v=221.66, x=0, y=0, w=0, z=0, d=0]",
 		"right 52.47 [h=114.637, v=221.66, x=0, y=0, w=0, z=0, d=0]",
-		"setrule 13.295, 0.398 [h=115.035, v=221.66, x=0, y=0, w=0, z=0, d=0]",
+		"setrule 13.295, .398 [h=115.035, v=221.66, x=0, y=0, w=0, z=0, d=0]",
 		"pop [h=61.768, v=221.66, x=0, y=0, w=0, z=0, d=0]",
-		"down 0.199 [h=61.768, v=221.859, x=0, y=0, w=0, z=0, d=0]",
-		"putrule 0.398, 53.267 [h=61.768, v=221.859, x=0, y=0, w=0, z=0, d=0]",
+		"down .199 [h=61.768, v=221.859, x=0, y=0, w=0, z=0, d=0]",
+		"putrule .398, 53.267 [h=61.768, v=221.859, x=0, y=0, w=0, z=0, d=0]",
 		"pop [h=61.768, v=218.472, x=0, y=0, w=0, z=0, d=0]",
 		"pop [h=61.768, v=218.472, x=0, y=0, w=0, z=0, d=0]",
 		"pop [h=61.768, v=218.472, x=0, y=0, w=0, z=0, d=0]",
@@ -550,7 +550,7 @@ TEST(DVIReaderTest, dvi_v3) {
 		"fontdef 7, 1274110073, cmr10 [h=25.088, v=43.042, x=0, y=0, w=0, z=0, d=1]",
 		"fontnum 7, cmr10 [h=25.088, v=43.042, x=0, y=0, w=0, z=0, d=1]",
 		"setchar0 118 [h=25.088, v=48.3, x=0, y=0, w=0, z=0, d=1]",
-		"right -0.277 [h=25.088, v=48.023, x=0, y=0, w=0, z=0, d=1]",
+		"right -.277 [h=25.088, v=48.023, x=0, y=0, w=0, z=0, d=1]",
 		"setchar0 101 [h=25.088, v=52.451, x=0, y=0, w=0, z=0, d=1]",
 		"setchar0 114 [h=25.088, v=56.353, x=0, y=0, w=0, z=0, d=1]",
 		"setchar0 116 [h=25.088, v=60.228, x=0, y=0, w=0, z=0, d=1]",
