@@ -2,7 +2,7 @@
 ** PSInterpreterTest.cpp                                                **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2019 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2020 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -38,7 +38,8 @@ class PSTestActions : public PSActions {
 		void fill (vector<double> &p) override            {print("fill", p);}
 		void gsave (vector<double> &p) override           {print("gsave", p);}
 		void grestore (vector<double> &p) override        {print("grestore", p);}
-		void grestoreall(std::vector<double> &p) override {print("grestoreall", p);}
+		void grestoreall (std::vector<double> &p) override{print("grestoreall", p);}
+		void image (std::vector<double> &p) override      {print("image", p);}
 		void initclip (vector<double> &p) override        {print("initclip", p);}
 		void lineto (vector<double> &p) override          {print("lineto", p);}
 		void moveto (vector<double> &p) override          {print("moveto", p);}
@@ -52,6 +53,7 @@ class PSTestActions : public PSActions {
 		void save(std::vector<double> &p) override        {print("save", p);}
 		void scale (vector<double> &p) override           {print("scale", p);}
 		void setblendmode (vector<double> &p) override    {print("setblendmode", p);}
+		void setcolorspace (vector<double> &p) override   {print("setcolorspace", p);}
 		void setcmykcolor (vector<double> &p) override    {print("setcmykcolor", p);}
 		void setdash (vector<double> &p) override         {print("setdash", p);}
 		void setgray (vector<double> &p) override         {print("setgray", p);}
@@ -61,6 +63,7 @@ class PSTestActions : public PSActions {
 		void setlinewidth (vector<double> &p) override    {print("setlinewidth", p);}
 		void setmatrix (vector<double> &p) override       {print("setmatrix", p);}
 		void setmiterlimit (vector<double> &p) override   {print("setmiterlimit", p);}
+		void setnulldevice (vector<double> &p) override   {print("setnulldevice", p);}
 		void setopacityalpha (vector<double> &p) override {print("setopacityalpha", p);}
 		void setshapealpha (vector<double> &p) override   {print("setshapealpha", p);}
 		void setrgbcolor (vector<double> &p) override     {print("setrgbcolor", p);}
@@ -101,7 +104,7 @@ TEST(PSInterpreterTest, gsave_grestore) {
 	actions.clear();
 
 	psi.execute("grestore ");
-	EXPECT_EQ(actions.result(), "setmatrix 1 0 0 1 0 0;applyscalevals 1 1 1;setlinewidth 1;setlinecap 0;setlinejoin 0;setmiterlimit 10;setrgbcolor 0 0 0;setdash 0;grestore;");
+	EXPECT_EQ(actions.result(), "setmatrix 1 0 0 1 0 0;applyscalevals 1 1 1;setlinewidth 1;setlinecap 0;setlinejoin 0;setmiterlimit 10;setcolorspace 0;setrgbcolor 0 0 0;setdash 0;grestore;");
 	actions.clear();
 
 	psi.execute("1 setlinecap 5 setmiterlimit 0 1 0 setrgbcolor gsave 0 setlinecap 10 setmiterlimit ");
@@ -109,7 +112,7 @@ TEST(PSInterpreterTest, gsave_grestore) {
 	actions.clear();
 
 	psi.execute("grestore ");
-	EXPECT_EQ(actions.result(), "setmatrix 1 0 0 1 0 0;applyscalevals 1 1 1;setlinewidth 1;setlinecap 1;setlinejoin 0;setmiterlimit 5;setrgbcolor 0 1 0;setdash 0;grestore;");
+	EXPECT_EQ(actions.result(), "setmatrix 1 0 0 1 0 0;applyscalevals 1 1 1;setlinewidth 1;setlinecap 1;setlinejoin 0;setmiterlimit 5;setcolorspace 0;setrgbcolor 0 1 0;setdash 0;grestore;");
 }
 
 
@@ -117,11 +120,11 @@ TEST(PSInterpreterTest, stroke_fill) {
 	PSTestActions actions;
 	PSInterpreter psi(&actions);
 	psi.execute("0 0 moveto 10 10 lineto 0 10 lineto closepath stroke ");
-	EXPECT_EQ(actions.result(), "setrgbcolor 0 0 0;newpath 0;moveto 0 0;lineto 10 10;lineto 0 10;closepath;stroke;");
+	EXPECT_EQ(actions.result(), "setcolorspace 0;setrgbcolor 0 0 0;newpath 0;moveto 0 0;lineto 10 10;lineto 0 10;closepath;stroke;");
 	actions.clear();
 
 	psi.execute("0 0 moveto 10 10 lineto 0 10 lineto closepath fill ");
-	EXPECT_EQ(actions.result(), "setrgbcolor 0 0 0;newpath 0;moveto 0 0;lineto 10 10;lineto 0 10;closepath;fill;");
+	EXPECT_EQ(actions.result(), "setcolorspace 0;setrgbcolor 0 0 0;newpath 0;moveto 0 0;lineto 10 10;lineto 0 10;closepath;fill;");
 }
 
 
