@@ -2,7 +2,7 @@
 ** SVGOutputTest.cpp                                                    **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2019 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2020 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -38,34 +38,34 @@ struct SVGOutputTest : public ::testing::Test {
 
 TEST_F(SVGOutputTest, defaults) {
 	SVGOutput out("SVGOutputTest.cpp", "");
-	EXPECT_EQ(out.filename(1, 1), "SVGOutputTest.svg");
-	EXPECT_EQ(out.filename(5, 9), "SVGOutputTest-5.svg");
-	EXPECT_EQ(out.filename(5, 10), "SVGOutputTest-05.svg");
-	EXPECT_EQ(out.filename(5, 256), "SVGOutputTest-005.svg");
+	EXPECT_EQ(out.filepath(1, 1).relative(), "SVGOutputTest.svg");
+	EXPECT_EQ(out.filepath(5, 9).relative(), "SVGOutputTest-5.svg");
+	EXPECT_EQ(out.filepath(5, 10).relative(), "SVGOutputTest-05.svg");
+	EXPECT_EQ(out.filepath(5, 256).relative(), "SVGOutputTest-005.svg");
 }
 
 
 TEST_F(SVGOutputTest, widthSpecifier) {
 	{
 		SVGOutput out("SVGOutputTest.cpp", "%f--%3p");
-		EXPECT_EQ(out.filename(5, 9), "SVGOutputTest--005.svg");
-		EXPECT_EQ(out.filename(54, 65), "SVGOutputTest--054.svg");
-		EXPECT_EQ(out.filename(543, 654), "SVGOutputTest--543.svg");
+		EXPECT_EQ(out.filepath(5, 9).relative(), "SVGOutputTest--005.svg");
+		EXPECT_EQ(out.filepath(54, 65).relative(), "SVGOutputTest--054.svg");
+		EXPECT_EQ(out.filepath(543, 654).relative(), "SVGOutputTest--543.svg");
 	}{
 		SVGOutput out("SVGOutputTest.cpp", "%f--%3p--%P");
-		EXPECT_EQ(out.filename(5, 9), "SVGOutputTest--005--9.svg");
-		EXPECT_EQ(out.filename(54, 65), "SVGOutputTest--054--65.svg");
-		EXPECT_EQ(out.filename(543, 654), "SVGOutputTest--543--654.svg");
+		EXPECT_EQ(out.filepath(5, 9).relative(), "SVGOutputTest--005--9.svg");
+		EXPECT_EQ(out.filepath(54, 65).relative(), "SVGOutputTest--054--65.svg");
+		EXPECT_EQ(out.filepath(543, 654).relative(), "SVGOutputTest--543--654.svg");
 	}{
 		SVGOutput out("SVGOutputTest.cpp", "%f--%3p--%3P");
-		EXPECT_EQ(out.filename(5, 9), "SVGOutputTest--005--009.svg");
-		EXPECT_EQ(out.filename(54, 65), "SVGOutputTest--054--065.svg");
-		EXPECT_EQ(out.filename(543, 654), "SVGOutputTest--543--654.svg");
+		EXPECT_EQ(out.filepath(5, 9).relative(), "SVGOutputTest--005--009.svg");
+		EXPECT_EQ(out.filepath(54, 65).relative(), "SVGOutputTest--054--065.svg");
+		EXPECT_EQ(out.filepath(543, 654).relative(), "SVGOutputTest--543--654.svg");
 	}{
 		SVGOutput out("SVGOutputTest.cpp", "%5f--%3p--%3P");
-		EXPECT_EQ(out.filename(5, 9), "SVGOutputTest--005--009.svg");
-		EXPECT_EQ(out.filename(54, 65), "SVGOutputTest--054--065.svg");
-		EXPECT_EQ(out.filename(543, 654), "SVGOutputTest--543--654.svg");
+		EXPECT_EQ(out.filepath(5, 9).relative(), "SVGOutputTest--005--009.svg");
+		EXPECT_EQ(out.filepath(54, 65).relative(), "SVGOutputTest--054--065.svg");
+		EXPECT_EQ(out.filepath(543, 654).relative(), "SVGOutputTest--543--654.svg");
 	}
 }
 
@@ -73,24 +73,24 @@ TEST_F(SVGOutputTest, widthSpecifier) {
 TEST_F(SVGOutputTest, expressions) {
 	{
 		SVGOutput out("SVGOutputTest.cpp", "no-macro");
-		EXPECT_EQ(out.filename(5, 9), "no-macro.svg");
-		EXPECT_EQ(out.filename(54, 65), "no-macro.svg");
-		EXPECT_EQ(out.filename(543, 654), "no-macro.svg");
+		EXPECT_EQ(out.filepath(5, 9).relative(), "no-macro.svg");
+		EXPECT_EQ(out.filepath(54, 65).relative(), "no-macro.svg");
+		EXPECT_EQ(out.filepath(543, 654).relative(), "no-macro.svg");
 	}{
 		SVGOutput out("SVGOutputTest.cpp", "%f--%(p-1)");
-		EXPECT_EQ(out.filename(5, 9), "SVGOutputTest--4.svg");
-		EXPECT_EQ(out.filename(54, 65), "SVGOutputTest--53.svg");
-		EXPECT_EQ(out.filename(543, 654), "SVGOutputTest--542.svg");
+		EXPECT_EQ(out.filepath(5, 9).relative(), "SVGOutputTest--4.svg");
+		EXPECT_EQ(out.filepath(54, 65).relative(), "SVGOutputTest--53.svg");
+		EXPECT_EQ(out.filepath(543, 654).relative(), "SVGOutputTest--542.svg");
 	}{
 		SVGOutput out("SVGOutputTest.cpp", "%f--%3(p-1)");
-		EXPECT_EQ(out.filename(5, 9), "SVGOutputTest--004.svg");
-		EXPECT_EQ(out.filename(54, 65), "SVGOutputTest--053.svg");
-		EXPECT_EQ(out.filename(543, 654), "SVGOutputTest--542.svg");
+		EXPECT_EQ(out.filepath(5, 9).relative(), "SVGOutputTest--004.svg");
+		EXPECT_EQ(out.filepath(54, 65).relative(), "SVGOutputTest--053.svg");
+		EXPECT_EQ(out.filepath(543, 654).relative(), "SVGOutputTest--542.svg");
 	}{
 		SVGOutput out("SVGOutputTest.cpp", "%f--%3(P+2p)");
-		EXPECT_EQ(out.filename(5, 9), "SVGOutputTest--019.svg");
-		EXPECT_EQ(out.filename(54, 65), "SVGOutputTest--173.svg");
-		EXPECT_EQ(out.filename(543, 654), "SVGOutputTest--1740.svg");
+		EXPECT_EQ(out.filepath(5, 9).relative(), "SVGOutputTest--019.svg");
+		EXPECT_EQ(out.filepath(54, 65).relative(), "SVGOutputTest--173.svg");
+		EXPECT_EQ(out.filepath(543, 654).relative(), "SVGOutputTest--1740.svg");
 	}
 }
 
@@ -99,16 +99,16 @@ TEST_F(SVGOutputTest, hashes) {
 	SVGOutput::HashTriple hashes("dvihash", "opthash", "cmbhash");
 	{
 		SVGOutput out("SVGOutputTest.cpp", "%f-%hd-x");
-		EXPECT_EQ(out.filename(1, 10), "SVGOutputTest--x.svg");
-		EXPECT_EQ(out.filename(1, 10, hashes), "SVGOutputTest-dvihash-x.svg");
+		EXPECT_EQ(out.filepath(1, 10).relative(), "SVGOutputTest--x.svg");
+		EXPECT_EQ(out.filepath(1, 10, hashes).relative(), "SVGOutputTest-dvihash-x.svg");
 	}{
 		SVGOutput out("SVGOutputTest.cpp", "%f-%hd-x-%hc%ho");
-		EXPECT_EQ(out.filename(1, 10), "SVGOutputTest--x-.svg");
-		EXPECT_EQ(out.filename(1, 10, hashes), "SVGOutputTest-dvihash-x-cmbhashopthash.svg");
+		EXPECT_EQ(out.filepath(1, 10).relative(), "SVGOutputTest--x-.svg");
+		EXPECT_EQ(out.filepath(1, 10, hashes).relative(), "SVGOutputTest-dvihash-x-cmbhashopthash.svg");
 	}{
 		SVGOutput out("SVGOutputTest.cpp", "%f-%hd%p%ho");
-		EXPECT_EQ(out.filename(1, 10), "SVGOutputTest-01.svg");
-		EXPECT_EQ(out.filename(1, 10, hashes), "SVGOutputTest-dvihash01opthash.svg");
+		EXPECT_EQ(out.filepath(1, 10).relative(), "SVGOutputTest-01.svg");
+		EXPECT_EQ(out.filepath(1, 10, hashes).relative(), "SVGOutputTest-dvihash01opthash.svg");
 	}
 }
 
@@ -117,12 +117,12 @@ TEST_F(SVGOutputTest, hashes_fail) {
 	SVGOutput::HashTriple hashes("dvihash", "opthash", "cmbhash");
 	{
 		SVGOutput out("SVGOutputTest.cpp", "%f-%h-x");
-		EXPECT_THROW(out.filename(1, 10), MessageException);
-		EXPECT_THROW(out.filename(1, 10, hashes), MessageException);
+		EXPECT_THROW(out.filepath(1, 10).relative(), MessageException);
+		EXPECT_THROW(out.filepath(1, 10, hashes).relative(), MessageException);
 	}{
 		SVGOutput out("SVGOutputTest.cpp", "%f-%hd-x-%ha%ho");
-		EXPECT_THROW(out.filename(1, 10), MessageException);
-		EXPECT_THROW(out.filename(1, 10, hashes), MessageException);
+		EXPECT_THROW(out.filepath(1, 10).relative(), MessageException);
+		EXPECT_THROW(out.filepath(1, 10, hashes).relative(), MessageException);
 	}
 }
 
@@ -151,11 +151,11 @@ TEST_F(SVGOutputTest, getPageStream) {
 
 TEST_F(SVGOutputTest, ignore) {
 	SVGOutput out("SVGOutputTest.cpp", "%x %y");
-	EXPECT_EQ(out.filename(5, 9), "SVGOutputTest-5.svg");
+	EXPECT_EQ(out.filepath(5, 9).relative(), "SVGOutputTest-5.svg");
 }
 
 
 TEST_F(SVGOutputTest, error) {
 	SVGOutput out("SVGOutputTest.cpp", "%(p/0)");
-	EXPECT_THROW(out.filename(5, 9), MessageException);
+	EXPECT_THROW(out.filepath(5, 9).relative(), MessageException);
 }
