@@ -2,7 +2,7 @@
  * Gregorio is a program that translates gabc files to GregorioTeX
  * This file provides functions for writing gabc from Gregorio structures.
  *
- * Copyright (C) 2006-2018 The Gregorio Project (see CONTRIBUTORS.md)
+ * Copyright (C) 2006-2019 The Gregorio Project (see CONTRIBUTORS.md)
  *
  * This file is part of Gregorio.
  * 
@@ -386,6 +386,12 @@ static void gabc_write_bar(FILE *f, gregorio_bar type)
     case B_DIVISIO_MAIOR_DOTTED:
         fprintf(f, ":?");
         break;
+    case B_DIVISIO_MINIMIS:
+        fprintf(f, "^");
+        break;
+    case B_DIVISIO_MINIMIS_HIGH:
+        fprintf(f, "^0");
+        break;
     default:
         /* not reachable unless there's a programming error */
         /* LCOV_EXCL_START */
@@ -542,6 +548,9 @@ static void gabc_write_gregorio_note(FILE *f, gregorio_note *note,
     case S_PUNCTUM_INCLINATUM_DESCENDENS:
         fprintf(f, "%c0", toupper((unsigned char)pitch_letter(note->u.note.pitch)));
         break;
+    case S_PUNCTUM_INCLINATUM_STANS:
+        fprintf(f, "%c2", toupper((unsigned char)pitch_letter(note->u.note.pitch)));
+        break;
     case S_PUNCTUM_INCLINATUM_DEMINUTUS:
         if (note->next) {
             fprintf(f, "%c~", toupper((unsigned char)pitch_letter(note->u.note.pitch)));
