@@ -19,6 +19,7 @@
 #include <kpathsea/config.h>
 #include <kpathsea/c-ctype.h>
 #include <kpathsea/c-pathch.h>
+#include <kpathsea/cnf.h>
 #include <kpathsea/expand.h>
 #include <kpathsea/getopt.h>
 #include <kpathsea/line.h>
@@ -39,10 +40,14 @@ string path_to_show = NULL;
 string var_to_value = NULL;
 string var_to_brace_value = NULL;
 
+/* Array/count of cnf lines from the command line. (--cnf-line) */
+static string *user_cnf_lines = NULL;
+static unsigned user_cnf_nlines = 0;
+
 /* Base resolution. (-D, -dpi) */
 unsigned dpi = 600;
 
-/* The engine name, for '$engine' construct in texmf.cnf.  (-engine) */
+/* The engine name, for `$engine' construct in texmf.cnf.  (-engine) */
 string engine = NULL;
 
 /* Interactively ask for names to look up?  (-interactive) */
@@ -474,6 +479,7 @@ to also use -engine, or nothing will be returned; in particular,\n\
 \n\
 -all                   output all matches, one per line (no effect with pk/gf).\n\
 [-no]-casefold-search  fall back to case-insensitive search if no exact match.\n\
+-cnf-line=STRING       parse STRING as a configuration file line.\n\
 -debug=NUM             set debugging flags.\n\
 -D, -dpi=NUM           use a base resolution of NUM; default 600.\n\
 -engine=STRING         set engine name to STRING.\n\
@@ -583,11 +589,20 @@ help_formats (kpathsea kpse, string *argv)
    option table in a variable `long_options'.  */
 #define ARGUMENT_IS(a) STREQ (long_options[option_index].name, a)
 
+/* If S is NULL or the empty string, abort.  */
+#define ENSURE_NONEMPTY_STRING(s) do {                                \
+  if (!(s) || ! *(s)) {                                               \
+    fprintf (stderr, "kpsewhich: empty argument for %s\n",            \
+             long_options[option_index].name);                        \
+    fputs ("Try `kpsewhich --help' for more information.\n", stderr); \
+    exit (EXIT_FAILURE); } } while (0)
+
 /* SunOS cc can't initialize automatic structs.  */
 static struct option long_options[]
   = { { "D",                    1, 0, 0 },
       { "all",                  0, (int *) &show_all, 1 },
       { "casefold-search",      0, 0, 0 },
+      { "cnf-line",             1, 0, 0 },
       { "debug",                1, 0, 0 },
       { "dpi",                  1, 0, 0 },
       { "engine",               1, 0, 0 },
@@ -627,7 +642,7 @@ read_command_line (kpathsea kpse, int argc, string *argv)
       break;
 
     if (g == '?')
-      exit (1);  /* Unknown option.  */
+      exit (EXIT_FAILURE);  /* Unknown option.  */
 
     assert (g == 0); /* We have no short option names.  */
 
@@ -637,6 +652,17 @@ read_command_line (kpathsea kpse, int argc, string *argv)
          (by default).  */
       xputenv ("texmf_casefold_search", "1");      
 
+    } else if (ARGUMENT_IS ("cnf-line")) {
+      if (user_cnf_lines == NULL) {
+        user_cnf_nlines = 1;
+        user_cnf_lines = xmalloc (sizeof (const_string));
+      } else {
+        user_cnf_nlines++;
+        user_cnf_lines = xrealloc (user_cnf_lines,
+                                   user_cnf_nlines * sizeof (const_string));
+      }
+      user_cnf_lines[user_cnf_nlines-1] = xstrdup (optarg);
+
     } else if (ARGUMENT_IS ("debug")) {
       kpse->debug |= atoi (optarg);
 
@@ -656,6 +682,7 @@ read_command_line (kpathsea kpse, int argc, string *argv)
       var_to_expand = optarg;
 
     } else if (ARGUMENT_IS ("format")) {
+      ENSURE_NONEMPTY_STRING (optarg);
       user_format_string = optarg;
 
     } else if (ARGUMENT_IS ("help")) {
@@ -685,9 +712,11 @@ read_command_line (kpathsea kpse, int argc, string *argv)
       progname = optarg;
 
     } else if (ARGUMENT_IS ("safe-in-name")) {
+      ENSURE_NONEMPTY_STRING (optarg);
       safe_in_name = optarg;
 
     } else if (ARGUMENT_IS ("safe-out-name")) {
+      ENSURE_NONEMPTY_STRING (optarg);
       safe_out_name = optarg;
 
     } else if (ARGUMENT_IS ("show-path")) {
@@ -698,14 +727,16 @@ read_command_line (kpathsea kpse, int argc, string *argv)
       str_list_add (&subdir_paths, optarg);
 
     } else if (ARGUMENT_IS ("var-brace-value")) {
+      ENSURE_NONEMPTY_STRING (optarg);
       var_to_brace_value = optarg;
 
     } else if (ARGUMENT_IS ("var-value")) {
+      ENSURE_NONEMPTY_STRING (optarg);
       var_to_value = optarg;
 
     } else if (ARGUMENT_IS ("version")) {
       puts (kpathsea_version_string);
-      puts ("Copyright 2019 Karl Berry & Olaf Weber.\n\
+      puts ("Copyright 2020 Karl Berry & Olaf Weber.\n\
 License LGPLv2.1+: GNU Lesser GPL version 2.1 or later <https://gnu.org/licenses/lgpl.html>\n\
 This is free software: you are free to change and redistribute it.\n\
 There is NO WARRANTY, to the extent permitted by law.\n");
@@ -716,10 +747,11 @@ There is NO WARRANTY, to the extent permitted by law.\n");
   }
 
   if (user_path && user_format_string) {
-    fprintf (stderr, "-path (%s) and -format (%s) are mutually exclusive.\n",
+    fprintf (stderr,
+            "kpsewhich: -path (%s) and -format (%s) are mutually exclusive.\n",
              user_path, user_format_string);
     fputs ("Try `kpsewhich --help' for more information.\n", stderr);
-    exit (1);
+    exit (EXIT_FAILURE);
   }
 
   if (optind == argc
@@ -728,7 +760,7 @@ There is NO WARRANTY, to the extent permitted by law.\n");
       && !safe_in_name && !safe_out_name) {
     fputs ("Missing argument. Try `kpsewhich --help' for more information.\n",
            stderr);
-    exit (1);
+    exit (EXIT_FAILURE);
   }
 }
 
@@ -742,6 +774,15 @@ init_more (kpathsea kpse)
   if (engine)
     kpathsea_xputenv (kpse, "engine", engine);
 
+  /* We want config lines from the command line to override config files.  */
+  if (user_cnf_lines) {
+    unsigned i;
+    for (i = 0; i < user_cnf_nlines; i++) {
+      kpathsea_cnf_line_env_progname (kpse, user_cnf_lines[i]);
+      free (user_cnf_lines[i]);
+    }
+  }
+
   /* Disable all mktex programs unless they were explicitly enabled on our
      command line.  */
 #define DISABLE_MKTEX(fmt) \
