@@ -18,13 +18,13 @@
    You should have received a copy of the GNU General Public License along
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
-
 #ifndef WRITEJP2_H
 #  define WRITEJP2_H
 
 #  include "image.h"
 
-void read_jp2_info(image_dict *, img_readtype_e);
+void read_jp2_info(image_dict *);
+void flush_jp2_info(image_dict *);
 void write_jp2(PDF, image_dict *);
 
 #endif                          /* WRITEJP2_H */
