@@ -18,14 +18,15 @@
    You should have received a copy of the GNU General Public License along
    with LuaTeX; if not, see <http://www.gnu.org/licenses/>. */
 
-
 #ifndef WRITEPNG_H
 #  define WRITEPNG_H
 
 #  include "image.h"
 
-void read_png_info(image_dict *, img_readtype_e);
+void read_png_info(image_dict *);
+void flush_png_info(image_dict *);
+
 void write_additional_png_objects(PDF);
 void write_png(PDF, image_dict *);
 
-#endif                          /* WRITEPNG_H */
+#endif
