@@ -2,7 +2,7 @@
 
 luainit.w
 
-Copyright 2006-2019 Taco Hoekwater <taco@@luatex.org>
+Copyright 2006-2020 Taco Hoekwater <taco@@luatex.org>
 
 This file is part of LuaTeX.
 
@@ -24,6 +24,7 @@ with LuaTeX; if not, see <http://www.gnu.org/licenses/>.
 #include "ptexlib.h"
 
 #include <kpathsea/c-stat.h>
+#include <kpathsea/cnf.h>
 
 #include "lua/luatex-api.h"
 
@@ -66,6 +67,7 @@ const_string LUATEX_IHELP[] = {
     "",
     "  The following regular options are understood: ",
     "",
+    "   --cnf-line =STRING            parse STRING as a configuration file line",
     "   --credits                     display credits and exit",
     "   --debug-format                enable format debugging",
     "   --draftmode                   switch on draft mode (generates no output PDF)",
@@ -183,6 +185,16 @@ static void prepare_cmdline(lua_State * L, char **av, int ac, int zero_offset)
 
 int kpse_init = -1;
 
+/*tex
+
+Array and count of values given with --cnf-line.
+
+*/
+static string *user_cnf_lines = NULL;
+static unsigned user_cnf_nlines = 0;
+
+
+
 string input_name = NULL;
 
 static string user_progname = NULL;
@@ -234,6 +246,7 @@ static struct option long_options[] = {
     {"interaction", 1, 0, 0},
     {"halt-on-error", 0, &haltonerrorp, 1},
     {"kpathsea-debug", 1, 0, 0},
+    {"cnf-line", 1,0 ,0},
     {"progname", 1, 0, 0},
     {"version", 0, 0, 0},
     {"credits", 0, 0, 0},
@@ -310,10 +323,12 @@ static void parse_options(int ac, char **av)
     opterr = 0;
 #ifdef LuajitTeX
     if ((strstr(argv[0], "luajittexlua") != NULL) ||
-        (strstr(argv[0], "texluajit") != NULL)) {
+        (strstr(argv[0], "texluajit") != NULL) ||
+        (strstr(argv[0], "texluahbjit") != NULL) ) {
 #else
     if ((strstr(argv[0], "luatexlua") != NULL) ||
-        (strstr(argv[0], "texlua") != NULL)) {
+        (strstr(argv[0], "texlua") != NULL) ||
+        (strstr(argv[0], "texluahb") != NULL)) {
 #endif
         lua_only = 1;
         luainit = 1;
@@ -359,6 +374,16 @@ static void parse_options(int ac, char **av)
             show_luahashchars = 1;
         } else if (ARGUMENT_IS("kpathsea-debug")) {
             kpathsea_debug |= atoi(optarg);
+        } else if (ARGUMENT_IS ("cnf-line")) {
+	  if (user_cnf_lines == NULL) {
+	    user_cnf_nlines = 1;
+	    user_cnf_lines = xmalloc (sizeof (const_string));
+	  } else {
+	    user_cnf_nlines++;
+	    user_cnf_lines = xrealloc (user_cnf_lines,
+				       user_cnf_nlines * sizeof (const_string));
+	  }
+	  user_cnf_lines[user_cnf_nlines-1] = xstrdup (optarg);
         } else if (ARGUMENT_IS("progname")) {
             user_progname = optarg;
         } else if (ARGUMENT_IS("jobname")) {
@@ -425,7 +450,7 @@ static void parse_options(int ac, char **av)
                  "the terms of the GNU General Public License, version 2 or (at your option)\n"
                  "any later version. For more information about these matters, see the file\n"
                  "named COPYING and the LuaTeX source.\n\n"
-                 "LuaTeX is Copyright 2019 Taco Hoekwater and the LuaTeX Team.\n");
+                 "LuaTeX is Copyright 2020 Taco Hoekwater and the LuaTeX Team.\n");
             /* *INDENT-ON* */
             uexit(0);
         } else if (ARGUMENT_IS("credits")) {
@@ -654,6 +679,7 @@ static int luatex_kpse_lua_find(lua_State * L)
         /*tex library not found in this path */
         return 1;
     }
+    recorder_record_input(filename);
     if (luaL_loadfile(L, filename) != 0) {
         luaL_error(L, "error loading module %s from file %s:\n\t%s",
             lua_tostring(L, 1), filename, lua_tostring(L, -1));
@@ -691,6 +717,7 @@ static int luatex_kpse_clua_find(lua_State * L)
             /*tex library not found in this path */
             return 1;
         }
+        recorder_record_input(filename);
         extensionless = strdup(filename);
         if (!extensionless) {
             /*tex allocation failure */
@@ -998,6 +1025,15 @@ void lua_initialize(int ac, char **av)
     putenv(LC_NUMERIC_C);
     /*tex this is sometimes needed */
     putenv(engine_luatex);
+    /*tex add user's cnf values*/
+    if (user_cnf_lines) {
+     unsigned i;
+     for (i = 0; i < user_cnf_nlines; i++) {
+      /* debug printf ("ucnf%d: %s\n", i, user_cnf_lines[i]); */
+      kpathsea_cnf_line_env_progname (kpse_def, user_cnf_lines[i]);
+      free (user_cnf_lines[i]);
+     }
+    }
     luainterpreter();
     /*tex init internalized strings */
     set_init_keys;
