@@ -478,23 +478,18 @@ static int lzlib_decompress(lua_State *L)
     {
         zs.next_out = (Bytef*)luaL_prepbuffer(&b);
         zs.avail_out = LUAL_BUFFERSIZE;
-
         /* bake some more */
-        ret = inflate(&zs, Z_FINISH);
-
+        ret = inflate(&zs, Z_NO_FLUSH);
         /* push gathered data */
         luaL_addsize(&b, LUAL_BUFFERSIZE - zs.avail_out);
-
         /* need dictionary? - no dictionary support here, so just quit */
         if (ret == Z_NEED_DICT)
             break;
-
         /* done processing? */
         if (ret == Z_STREAM_END)
             break;
-
         /* error condition? */
-        if (ret != Z_BUF_ERROR)
+        if (ret != Z_OK)
             break;
     }
 
@@ -574,7 +569,7 @@ LUALIB_API int luaopen_zlib(lua_State *L)
     ** Stack:
     */
     lua_newtable(L);
-    luaL_openlib(L, NULL, zlib, 0);
+    luaL_openlib(L, "zlib", zlib, 1);
 
     /*
     ** Stack: zlib table
